"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
exports.createSharedStateHook = (initialState) => {
    let state = initialState;
    let handlers = [];
    const hook = (selectorFn, isEqualFn) => {
        const select = selectorFn !== undefined ? selectorFn : (o) => o;
        const [selection, setSelection] = react_1.useState(() => select(state));
        const selectionRef = react_1.useRef(selection);
        const selectorRef = react_1.useRef(select);
        const isEqualRef = react_1.useRef(isEqualFn);
        selectorRef.current = select; // allow new selector fn on each render
        isEqualRef.current = isEqualFn; // allow new isEqual fn on each render
        react_1.useEffect(() => {
            const handler = (newState) => {
                const nextSelection = selectorRef.current(newState);
                const currentIsEqualFn = isEqualRef.current;
                const previousSelection = selectionRef.current;
                const isEqual = currentIsEqualFn !== undefined
                    ? currentIsEqualFn(previousSelection, nextSelection)
                    : previousSelection === nextSelection;
                if (!isEqual) {
                    selectionRef.current = nextSelection;
                    setSelection(nextSelection);
                }
            };
            handlers.push(handler);
            return () => {
                handlers = handlers.filter(fn => fn !== handler);
            };
        }, []);
        return selection;
    };
    const setter = newState => {
        state = newState;
        handlers.forEach(fn => fn(state));
    };
    return [hook, setter];
};
